/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.NahkampfwaffeXmlTags.*;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class Nahkampfwaffe implements Comparable< Nahkampfwaffe > {

    /** Zum Sortieren der Waffen nach Name. */
    private static final Collator COLLATOR = Collator.getInstance();

    /** Der Bruchfaktor der Waffe. */
    private int bf;

    /** Distanzklasse Handegemenge */
    private boolean dkH;

    /** Distanzklasse Nahkampf */
    private boolean dkN;

    /** Distanzklasse Stangenwaffe */
    private boolean dkS;

    /** Der Name der Waffe. */
    private String name;

    /** Die Id der Waffe. */
    private final String id;

    /** Der Initiativemodifikator der Waffe. */
    private int ini;

    /** Der Schadensschritt fuer die TP/KK. */
    private int schadensschritt;

    /** Der Schwellenwert fuer TP/KK. */
    private int schwellenwert;

    /** Das Talent mit dem die Waffe ueblicherweise gefuehrt wird. */
    private Kampftechnik talent;

    /** Alle Talente mit denen die Waffe gefuehrt werden kann. */
    private final Set< Kampftechnik > talente;

    /** Die Formel fuer die Trefferpunkte. */
    private Tp tp;

    /** Der Waffenmodifikator auf Attacke. */
    private int wmAt;

    /** Der Waffenmodifikator auf Parade */
    private int wmPa;

    public Nahkampfwaffe( final String dieId ) {

        bf = 1;

        dkH = false;

        dkN = true;

        dkS = false;

        id = dieId;

        ini = 0;

        if ( I18n.hatLokalisierungstext( id ) ) {

            name = I18n.i18n( id );

        } else {

            name = id;
        }

        schadensschritt = 4;

        schwellenwert = 11;

        talente = new TreeSet< Kampftechnik >();

        tp = new Tp( "W+4" );

        wmAt = 0;

        wmPa = 0;
    }

    /** Sortiert die Waffen alphabetisch nach Namen. */
    public int compareTo( final Nahkampfwaffe o ) {

        int ergebnis = COLLATOR.compare( name, o.getName() );

        if ( 0 == ergebnis ) {

            ergebnis = COLLATOR.compare( id, o.getId() );
        }

        return ergebnis;
    }

    @Override
    public boolean equals( final Object obj ) {

        if ( null == obj ) {
            return false;
        }

        if ( obj.getClass().equals( getClass() ) ) {

            return ( 0 == compareTo( (Nahkampfwaffe) obj ) );

        } else {

            return false;
        }
    }

    /** @return den Bruchfaktor der Waffe. */
    public final int getBf() {
        return bf;
    }

    /** @return Die Id der Waffe. */
    public final String getId() {
        return id;
    }

    /** @return Den Initiativemodifikator der Waffe. */
    public final int getIni() {
        return ini;
    }

    /** @return den Namen der Waffen. */
    public final String getName() {
        return name;
    }

    /** @return den Schadensschritt fuer die TP/KK. */
    public final int getSchadensschritt() {
        return schadensschritt;
    }

    /** @return den Schwellenwert fuer die TP/KK. */
    public final int getSchwellenwert() {
        return schwellenwert;
    }

    /** @return das Talent mit dem die Waffe ueblicherweise gefuehrt wird. */
    public final Kampftechnik getTalent() {
        return talent;
    }

    /** @return Alle Talente mit denen die Waffe gefuehrt werden kann. */
    public final List< Kampftechnik > getTalente() {
        return new ArrayList< Kampftechnik >( talente );
    }

    /** @return Die Formel fuer die TP. */
    public final Tp getTp() {
        return tp;
    }

    /** @return den Waffenmodifikator auf Attacke. */
    public final int getWmAt() {
        return wmAt;
    }

    /** @return den Waffenmodifikator auf Parade. */
    public final int getWmPa() {
        return wmPa;
    }

    /**
     * @return {@code true}, wenn es sich bei der Kampftechnik um eine handelt,
     *         mit der die Waffe, ohne Abzuege, gefuehrt werden kann, sonst
     *         {@code false}. Es gibt noch die mit den Haupttalenten verwandten
     *         Talente. Fuehrt man die Waffe mit diesen Talenten erhaelt man
     *         Abzuege von -2/-3.
     */
    public final boolean hatHaupttalent( final Kampftechnik technik ) {
        return talente.contains( technik );
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Handgemenge
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkH() {
        return dkH;
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Nahkampf
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkN() {
        return dkN;
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Stangenwaffe
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkS() {
        return dkS;
    }

    /** @param neuerBf Der neue Bruchfaktor der Waffe. */
    public final void setBf( final int neuerBf ) {
        bf = neuerBf;
    }

    /**
     * @param handgemenge {@code true}, wenn die Waffe in der Distanzklasse
     *            Handgemenge gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkH( final boolean handgemenge ) {
        dkH = handgemenge;
    }

    /**
     * @param nahkmapf {@code true}, wenn die Waffe in der Distanzklasse
     *            Nahkampf gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkN( final boolean nahkampf ) {
        dkN = nahkampf;
    }

    /**
     * @param stangenwaffe {@code true}, wenn die Waffe in der Distanzklasse
     *            Handgemenge gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkS( final boolean stangenwaffe ) {
        dkS = stangenwaffe;
    }

    /** @param neueIni Der neue Initiativemodifikator der Waffe. */
    public final void setIni( final int neueIni ) {
        ini = neueIni;
    }

    /**
     * @param neueName fuer die Waffe.
     */
    public final void setName( final String neuerName ) {

        for ( final Talent t : talente ) {

            t.entferneSpezialisierung( name );
        }

        name = neuerName;

        for ( final Talent t : talente ) {

            t.einfuegenSpezialisierung( name );
        }
    }

    /**
     * @param neuerSchadensschritt fuer die TP/KK.
     */
    public final void setSchadensschritt( final int neuerSchadensschritt ) {
        schadensschritt = neuerSchadensschritt;
    }

    /** @param neuerSchwellenwert fuer die TP/KK. */
    public final void setSchwellenwert( final int neuerSchwellenwert ) {
        schwellenwert = neuerSchwellenwert;
    }

    /**
     * @param haupttalent
     *            Das Talent mit dem die Waffe üblicherweise geführt wird.
     */
    public final void setTalent( final Kampftechnik haupttalent ) {

        if ( null != talent ) {
            talente.remove( talent );

            talent.entferneSpezialisierung( name );
        }

        talent = haupttalent;

        talent.einfuegenSpezialisierung( name );

        talente.add( talent );
    }

    /**
     * @param neueTalente
     *            Die Talente mit denen die Waffe gefuehrt werden kann. Das
     *            Haupttalent ({@link #setTalent(Collection)}) muss hier nicht
     *            noch mal angegeben werden, kann aber.
     *
     */
    public final void setTalente( final Collection< Kampftechnik > neueTalente ) {

        for ( final Talent t : talente ) {

            t.entferneSpezialisierung( name );
        }

        talente.clear();

        if ( null != talent ) {

            talente.add( talent );
        }

        talente.addAll( neueTalente );

        for ( final Talent t : talente ) {

            t.einfuegenSpezialisierung( name );
        }
    }

    /** @param neueTp fuer die Waffe. */
    public final void setTp( final Tp neueTp ) {
        tp = neueTp;
    }

    /** @param modifikator auf die Attacke. */
    public final void setWmAt( final int modifikator ) {
        wmAt = modifikator;
    }

    /** @param modifikator auf die Parade. */
    public final void setWmPa( final int modifikator ) {
        wmPa = modifikator;
    }

    @Override
    public final String toString() {
        return name;
    }

    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_WAFFE );

        element.setAttribute( ATTR_BF, String.valueOf( bf ) );

        element.setAttribute( ATTR_DK_H, String.valueOf( dkH ) );

        element.setAttribute( ATTR_DK_N, String.valueOf( dkN ) );

        element.setAttribute( ATTR_DK_S, String.valueOf( dkS ) );

        element.setAttribute( ATTR_ID, id );

        element.setAttribute( ATTR_INI, String.valueOf( ini ) );

        if ( I18n.hatLokalisierungstext( id ) ) {

            if ( ! I18n.i18n( id ).equals( name ) ) {

                element.setAttribute( ATTR_NAME, name );
            }

        } else {

            element.setAttribute( ATTR_NAME, name );
        }

        element.setAttribute( ATTR_SCHADENSSCHRITT,
                              String.valueOf( schadensschritt ) );

        element.setAttribute( ATTR_SCHWELLENWERT,
                              String.valueOf( schwellenwert ) );

        element.setAttribute( ATTR_TALENT, talent.getId() );

        element.setAttribute( ATTR_TP, tp.toString() );

        element.setAttribute( ATTR_WM_AT, String.valueOf( wmAt ) );

        element.setAttribute( ATTR_WM_PA, String.valueOf( wmPa ) );

        for ( final Talent talent : talente ) {

            final IXMLElement talentElement = new XMLElement( TAG_TALENT );

            talentElement.setContent( talent.getId() );

            element.addChild( talentElement );
        }

        return element;
    }
}
