/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.*;

import java.text.Collator;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/** Die moeglichen Ruestungen die ein Held tragen kann. */
public class Ruestung implements Comparable< Ruestung >{

    private static final Collator COLLATOR = Collator.getInstance();

    public static final String KETTENHEMD =
        I18n.i18n( "Ruestung.Kettenhemd" );

    public static final String LANGES_KETTENHEMD =
        I18n.i18n( "Ruestung.LangesKettenhemd" );

    /** Die Behinderung. */
    private int be;

    /** Die Id zur Identifikation der Ruestung. */
    private final String id;

    /**
     * Gibt bei einer Zusatzruestung an wie hoch der Torso-RS sein muss, damit
     * der RS der Zusatzruestung angerechnet wird.
     */
    private int mindestTorsoRs;

    /** Der Name der Ruestung. */
    private String name;

    /** Der Ruestungsschutz. */
    private int rs;

    /** Gibt an ob es sich um eine Torsoruestung oder eine Zusatzruestung. */
    private boolean zusatzruestung;

    public Ruestung( final String dieId ) {

        id = dieId;

        be = 1;

        mindestTorsoRs = 0;

        if ( I18n.hatLokalisierungstext( id ) ) {

            name = I18n.i18n( id );

        } else {

            name = id;
        }

        rs = 1;

        zusatzruestung = false;
    }

    /**
     * Sortier die Ruestungen alphabetisch nach Namen.
     */
    public final int compareTo( final Ruestung o ) {

        int ergebnis = COLLATOR.compare( getName(), o.getName() );

        if ( 0 == ergebnis ) {

            ergebnis = COLLATOR.compare( getId(), o.getId() );
        }

        return ergebnis;
    }

    @Override
    public boolean equals( final Object obj ) {
        if ( null == obj ) {
            return false;
        }

        if ( getClass().equals( obj.getClass() ) ) {

            return ( 0 == compareTo( (Ruestung) obj ) );

        } else {

            return false;
        }
    }

    /**
     * @return die Behinderung
     */
    public final int getBe() {
        return be;
    }

    /**
     * @return die Id.
     */
    public final String getId() {
        return id;
    }

    /**
     * @return Liefert bei einer Zusatzruestung, den RS den man am Torso
     *         besitzen muss, damit der RS der Zusatzruestung angerechnet
     *         wird.
     */
    public int getMindestTorsoRs() {
        return mindestTorsoRs;
    }

    /** @return der Name der Ruestung. */
    public final String getName() {
        return name;
    }

    /** @return den Ruestungsschutz. */
    public final int getRs() {
        return rs;
    }

    /**
     * @return {@code true}, wenn es sich um einen Helm oder Ruestzeug handelt,
     *         {@code false}, wenn es sich um eine Torso-Ruestung handelt.
     */
    public final boolean istZusatzruestung() {
        return zusatzruestung;
    }

    /**
     * Aendert die Behinderung der Ruestung.
     *
     * @param neueBe Die neue Behinderung.
     */
    public final void setBe( final int neueBe ) {
        be = neueBe;
    }

    /**
     * @param torsoRs bei einer Zusatzruestung der RS, den man am Torso haben
     *        muss, um Nutzen aus der Zusatzruestung zu ziehen.
     */
    public void setMindestTorsoRs( final int torsoRs ) {
        mindestTorsoRs = torsoRs;
    }

    /**
     * Aendert den Namen der Ruestung.
     *
     * @param neuerName fuer die Ruestung.
     */
    public final void setName( final String neuerName ) {
        name = neuerName;
    }

    /**
     * Aendert den Ruestungsschutz der Ruestung.
     *
     * @param neuerRs Der neue Ruestungschutz.
     */
    public final void setRs( final int neuerRs ) {
        rs = neuerRs;
    }

    /**
     * @param istZusatzruestung
     *            {@code true}, wenn es sich um einen Helm oder Ruestzeug
     *            handelt, {@code false}, wenn es sich um eine Torso-Ruestung
     *            handelt.
     */
    public final void setZusatzruestung( final boolean istZusatzruestung ) {
        zusatzruestung = istZusatzruestung;
    }

    @Override
    public String toString() {
        return name;
    }

    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_RUESTUNG );

        element.setAttribute( ATTR_BE, String.valueOf( be ) );

        element.setAttribute( ATTR_ID, id );

        if ( zusatzruestung ) {

            element.setAttribute( ATTR_MINDEST_TORSO_RS,
                                  String.valueOf( mindestTorsoRs ) );
        }

        if ( ! I18n.hatLokalisierungstext( id ) ) {

            // In der Lokalisierungsdatei befindet sich kein Name fuer die
            // Ruestung. Der Name muss extra gespeichert werden.
            element.setAttribute( ATTR_NAME, name );

        } else {

            final String lokalisierung = I18n.i18n( id );

            if ( ! lokalisierung.equals( name ) ) {

                // Der Benutzer hat einen anderen Namen fuer die Ruestung
                // eingegeben. Dieser Name muss gespeichert werden.
                element.setAttribute( ATTR_NAME, name );
            }
        }

        element.setAttribute( ATTR_RS, String.valueOf( rs ) );

        element.setAttribute( ATTR_ZUSATZRUESTUNG,
                              String.valueOf( zusatzruestung ) );

        return element;
    }
}
