/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.heldio;

import java.util.LinkedList;
import java.util.Queue;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Ein Klasse die dabei Hilft die Tabelle mit den Talentwerten, beim Rtf-Export,
 * zu fuellen. Sie dient dazu festzustellen welche Talente noch in die Tabelle
 * geschrieben werden muessen. Dazu enthaelt ein Objekt dieser Klasse zwei
 * Listen vor deren Inhalt jeweils nacheinander abgearbeitet wird. Einmal eine
 * Liste mit den Talentgruppen die noch abzuarbeiten sind und die Liste mit den
 * Talenten die in der aktuell bearbeiteten Talentgruppe noch in die Tabelle
 * einzutragen sind. Mit {@link #getTalentgruppe()} kann man die aktuelle
 * bearbeitete Talentgruppe erfragen. Die Methode {@link #getNaechstesTalent()}
 * liefert dann das naechste Talent der Talentgruppe und entfernt es aus der
 * Liste der Talente. Der naechste Aufruf von {@code getNaechstesTalent} liefert
 * dann das naechste Talent usw. bis alle Talente abgearbeitet sind. Dies kann
 * mit {@link #hatNaechstesTalent()} ueberprueft werden. Ist eine Talentgruppe
 * vollstaendig durchlaufen kann man mit {@link #naechsteTalentgruppe()} zur
 * naechsten Talentgruppe uebergehen. Mit {@link #hatTalentgruppe()} kann
 * geprueft werden ob es noch eine naechste Talentgruppe gibt. Es muss erst
 * {@code #naechsteTalentgruppe()} aufgerufen werden, dann
 * {@code #hatTalentgruppe()}.
 * <p>
 * Die Klasse liefert alle Talentgruppen in der fuer den Talentbogen ueblichen
 * Reihenfolge, allerdings ohne die Talentgruppe Kampf. Die Talente werden
 * alphabetisch sortiert geliefert.
 *
 * @author Bernhard Jung
 */
class TalenteTabelleDaten {

    /** Der Held dessen Talente in die Tabelle eingefuegt werden. */
    public final Held held;

    /**
     * Die Talentgruppen die noch in die Tabelle eingetragen werden muessen.
     */
    public final Queue< Talentgruppe > talentgruppen;

    /**
     * Die Talente der aktuellen Talentgruppe die noch in die Tabelle
     * eingetragen werden muessen.
     */
    public final Queue< Talent > talente;

    /**
     * @param derHeld dessen Daten in die Tabelle eingetragen werden sollen.
     */
    TalenteTabelleDaten( final Held derHeld ) {

        held = derHeld;

        talentgruppen = new LinkedList< Talentgruppe >();
        for ( final Talentgruppe talentgruppe
                : Talentgruppe.getTalentgruppenAlle() ) {

            // Die Talentgruppe Kampf wird von HeldExportRtf extra abgehandelt
            if ( Talentgruppe.KAMPF != talentgruppe ) {

                talentgruppen.add( talentgruppe );
            }
        }

        talente = new LinkedList< Talent >();

        sucheTalentgruppeMitTalenten();
    }

    /**
     * Sucht die naechste Talentgruppe deren Talente in die Tabelle
     * einzutragen sind.
     */
    public final void naechsteTalentgruppe() {

        // Sicherheitshalber die Talentliste leeren.
        talente.clear();

        // Die aktuelle Talentgruppe ist abgearbeitet, es soll zur folgenden
        // umgeschaltet werden.
        talentgruppen.poll();

        sucheTalentgruppeMitTalenten();
    }

    /**
     * Durchlaeuft die Liste der Talentgruppen und sucht die erste, in welcher
     * der Held Talente besitzt. Die Talente werden in {@link #talente}
     * eingetragen.
     */
    private final void sucheTalentgruppeMitTalenten() {

        // So lange noch eine nicht bearbeitete Talentgruppe da ist
        // wird geprueft ob der Held Talente aus der Talentgruppe hat.
        while ( ! talentgruppen.isEmpty() ) {

            final Talentgruppe talentgruppe = talentgruppen.peek();

            talente.addAll( held.getTalente( talentgruppe ) );

            if ( talente.isEmpty() ) {
                // Der Held besitzt keine Talente in der Talentgruppe, die
                // Gruppe kann uebergangen werden.
                talentgruppen.poll();

            } else {
                // Es gibt in dieser Talentgruppe Talente die in die Tabelle
                // eingetragen werden muessen. Die naechste Talentgruppe ist
                // gefunden.
                break;

            }
        }
    }

    /**
     * @see #hatNaechstesTalent()
     * @return liefert das naechste Talent welches in die Tabelle einzutragen
     *      ist. Wenn kein Talent mehr uebrig ist wird {@code null} geliefert.
     */
    public final Talent getNaechstesTalent() {
        return talente.poll();
    }

    /**
     * @see #hatTalentgruppe()
     * @return die Talentgruppe deren Talente von {@link #getNaechstesTalent()}
     *      geliefert werdern. Die Methode liefert {@code null} wenn alle
     *      Talentgruppen abgearbeitet wurden.
     */
    public final Talentgruppe getTalentgruppe() {
        return talentgruppen.peek();
    }

    /**
     * @return {@code true} wenn es in der aktuellen Talentgruppe noch ein
     *      Talent gibt, das in die Tabelle eingetragen werden muss, sonst
     *      {@code false}.
     */
    public final boolean hatNaechstesTalent() {
        return ! talente.isEmpty();
    }

    /**
     * @return {@code true} wenn es noch eine Talentgruppe gibt, deren Talente
     *      in die Tabelle eingetragen werden muessen, sonst {@code false}.
     */
    public final boolean hatTalentgruppe() {
        return ! talentgruppen.isEmpty();
    }
}
