/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basiswerte;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;

/**
 * Ein Basiswert eines Helden.
 *
 * @author Bernhard Jung
 *
 */
public class BasiswertWert {

    /** Der Basiswert um den es sich handelt. */
    private final Basiswert basiswert;

    /** Zusaetzliche Bonuspunkte wie sie zB durch Grosse Meditation entstehen.*/
    private int bonus;

    /** Der Held, dem der Basiswert gehoert. */
    private final Held held;

    /**
     * Die AP-Kosten, die aufgebracht wurden um den Basiswert zu steigern.
     * Die Kosten fuer den i-ten zugekauften Punkte stehen an Position i.
     */
    private final List< Integer > kosten;

    /** Gibt an, ob der Held eine spezielle Erfahrung erhalten hat. */
    private boolean spezielleErfahrung;

    /** Die Punkte die zum Startwert des Basiswerts hinzugekauft wurden. */
    private int zukauf;

    /**
     * Die vorgegebenen Punkte die zum Startwert des Basiswerts hinzugekauft
     * wurden. Das sind die zugekauften Punkte, die der Held direkt nach dem
     * Laden besitzt.
     */
    private int zukaufVorgegeben;

    /**
     * @param derHeld
     *            dem der Basiswert gehoert.
     * @param derBasiswert
     *            um den es sich handelt.
     */
    public BasiswertWert(
            final Held derHeld,
            final Basiswert derBasiswert ) {

        if ( ( null == derHeld ) || ( null == derBasiswert ) ) {
            throw new IllegalArgumentException(
                    "Es wurde null als Argument uebergeben." );
        }

        basiswert = derBasiswert;

        bonus  = 0;

        held = derHeld;

        kosten = new ArrayList< Integer >();
        kosten.add( 0 );

        spezielleErfahrung = false;

        zukauf = 0;

        zukaufVorgegeben = 0;
    }

    /**
     * @param punkte
     *            die der Held zusaetzliche, zB durch Grosse Meditation,
     *            erhalten hat. Es handelt sich hier nicht um Punkte die
     *            regulaer durch AP hinzugekauft wurden.
     */
    public final void addiereBonuspunkte( final int punkte ) {

        if ( punkte != 0 ) {

            bonus += punkte;

            held.notifyObservers( basiswert );
        }
    }

    /**
     * @return die Punkte die alleine durch die Berechnung der Formel entstehen.
     */
    public final int getBasisstufe() {
        return basiswert.getBasisstufe( held );
    }

    /**
     * @return den Basiswert fuer den der Wert steht.
     */
    public final Basiswert getBasiswert() {
        return basiswert;
    }

    /**
     * @return zusaeztliche Bonuspunkte, die zB durch Grosse Meditation
     *         entstanden sind.
     */
    public final int getBonus()  {
        return bonus;
    }

    /**
     * @return der Held dem der Wert gehoert.
     */
    public final Held getHeld() {
        return held;
    }

    /**
     * @return den Namen des Basiswerts.
     */
    public final String getName() {
        return basiswert.getName();
    }

    /**
     * @return die Anzahl Puntke die maximal zur Startstufe des Basiswerts
     *         hinzugekauft werden koennen.
     */
    public final int getMaxZukauf() {
        return basiswert.getMaxZukauf( held );
    }

    /**
     * @return die Modifikation auf den Basiswert durch Vor- und Nachteile.
     */
    public final int getModifikation() {
        return basiswert.getModifikation( held );
    }

    /**
     * @return die AP-Kosten, die der Held aufbringen muss um den Wert um einen
     *         Punkt anzuheben.
     */
    public final int getSteigerungskosten() {

        int steigerungsfaktor = basiswert.getSteigerungsspalte();

        if ( spezielleErfahrung ) {

            steigerungsfaktor = Lernmethode.SPEZIELLE_ERFAHRUNG
                                .bestimmeSteigerungsfaktor( basiswert );
        }

        return SKT.getKosten( ( 1 + zukauf ), steigerungsfaktor );
    }

    /**
     * @return die Stufe des Helden in diesem Basiswert.
     */
    public final int getStufe() {
        return basiswert.getBasisstufe( held )
             + basiswert.getModifikation( held )
             + zukauf
             + bonus ;
    }

    /**
     * @return die Anzahl Punkte die zur Startstufe des Basiswerts hinzugekauft
     *         worden sind.
     */
    public final int getZukauf() {
        return zukauf;
    }

    /**
     * @return {@code true}, wenn der Held eine spezielle Erfahrung auf die
     *         Eigenschaft erhalten hat, sonst {@code false}.
     */
    public final boolean hatSpezielleErfahrung() {
        return spezielleErfahrung;
    }

    /**
     * Prueft ob der Held, der den Wert bestizt alle Voraussetzungen fuer ihn
     * erfuellt.
     */
    public final KonfliktBeschreibung pruefen() {
        return basiswert.pruefen( held, this );
    }

    /* *
     * Senkt die zugekauften Punkte um einen Punkt und schreibt dem Held die
     * freigewordenen AP gut. Wenn die zugekauften Punkte schon 0 sind, so macht
     * die Methode nichts.
     * /
    public final void senken() {

        if ( zukauf > 0 ) {

            held.addiereEingesetzteAp( - kosten.remove( zukauf ) );

            --zukauf;

            held.notifyObservers( basiswert );
        }
    }
*/
    /**
     * Stellt den Bonus auf den Basiswert ein.
     *
     * @param derBonus
     *            auf den Basiswert.
     */
    public final void setBonus( final int derBonus ) {

        if ( derBonus != bonus ) {

            bonus = derBonus;

            held.notifyObservers( basiswert );
        }
    }

    /**
     * @param erfahrung
     *            {@code true}, wenn der Held eine spezielle Erfahrung auf die
     *            Eigenschaft erhalten hat, sonst {@code false}.
     */
    public final void setSpezielleErfahrung( final boolean erfahrung ) {

        if ( erfahrung != spezielleErfahrung ) {

            spezielleErfahrung = erfahrung;

            held.notifyObservers( basiswert );
        }
    }

    /**
     * Stellt die Zugekauften Punkte fuer den Basiswert ein und berechnet dem
     * Helden die dafuer entstehenden Kosten.
     *
     * @param punkte die der Held zugekauft hat.
     */
    public final void setZukauf( final int punkte ) {

        if ( punkte > zukauf ) {

            int summe = 0;

            for( ; zukauf < punkte ; ++zukauf ) {

                final int apKosten;

                if ( zukauf < zukaufVorgegeben ) {

                    apKosten = 0;

                } else {

                    apKosten = getSteigerungskosten();

                }

                spezielleErfahrung = false;

                kosten.add( apKosten );

                summe += apKosten;
            }

            held.addiereApEingesetzt( summe );

            held.notifyObservers( basiswert );

        } else if ( punkte < zukauf ){

            int summe = 0;

            for ( ; zukauf > punkte ; --zukauf ) {

                summe += kosten.remove( zukauf );
            }

            held.addiereApEingesetzt( -summe );

            held.notifyObservers( basiswert );
        }
    }

    /**
     * Stellt die zugekauften Punkte ein, die der Held direkt nach dem Laden des
     * Helden besitzt. Stellt die eigentlichen zugekauften Punkte auf den selben
     * Wert ein.
     *
     * @param punkte
     *            die der Held zum Basiswert zugekauft hat.
     */
    public final void setZukaufVorgegeben( final int punkte ) {

        if ( punkte > zukauf ) {

            for( ; zukauf < punkte ; ++zukauf ) {

                kosten.add( 0 );
            }

            zukaufVorgegeben = punkte;

            held.notifyObservers( basiswert );

        } else if ( punkte < zukauf ){

            for ( ; zukauf > punkte ; --zukauf ) {

                kosten.remove( zukauf );
            }

            zukaufVorgegeben = punkte;

            held.notifyObservers( basiswert );
        }

    }
    /* *
     * Steigert den Basiswert um einen Punkt, in dem die zugekauften Punkte
     * angehoben werden. Die Kosten werden dem Helden in Rechnung gestellt.
     * /
    public final void steigern() {

        final int apKosten = getSteigerungskosten();

        kosten.add( apKosten );

        held.addiereEingesetzteAp( apKosten );

        ++zukauf;

        spezielleErfahrung = false;

        held.notifyObservers( basiswert );
    }
    */

}
