/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Ein Talentbonus der von Rasse, Kultur oder Profession vergeben wird und bei
 * dem ein Stufenbonus frei auf mehrere Talente verteilt werden darf.
 * Er hat die Form '4 Punkte auf Oloarkh, Ologhaijan oder Thorwalsch verteilen'
 * @author Bernhard Jung
 *
 */
public class TalentbonusFrei implements Talentbonus {

    /** Die Talentpunkte die verteilt werden sollen. */
    private final int stufenBonus;

    /** Die Talente auf die der Bonus verteilt werden soll. */
    private final List< Talent > talente;

    /**
     * Eine Beschreibung des Bonus wie '4 Punkte auf Schwerter und/oder Saebel
     * verteilen'
     */
    private final String beschreibungString;

    public TalentbonusFrei(
            final int derStufenbonus,
            final List< Talent > dieTalente ) {

        stufenBonus = derStufenbonus;

        talente = new ArrayList< Talent >( dieTalente );
        Collections.sort( talente );

        beschreibungString = String.format(
                I18n.i18n( "TalentBonusFrei" ),
                stufenBonus,
                Strings.erzeugeAufzaehlungMitUndOder( talente ) );
    }

    public int compareTo( final Talentbonus o ) {

        return getTalent().compareTo( o.getTalent() );
    }

    /**
     * @return die GP-Kosten die einem Baustein durch diesen Bonus entstehen.
     */
    public double getGpKosten() {

        double kosten = 0.0;

        for ( final Talent talent : talente ) {

            if ( ! talent.istBasisTalent() ) {

                kosten += talent.getSteigerungsfaktor();
            }

            kosten += ( stufenBonus * talent.getSteigerungsfaktor() );
        }

        return ( kosten / 10.0 ) / talente.size();
    }

    /**
     * @return der Stufenbonus den es auf die Talente zu verteilen gilt.
     */
    public final int getStufenBonus() {
        return stufenBonus;
    }

    /** {@inheritDoc} */
    public final Talent getTalent() {
        return talente.get( 0 );
    }

    /**
     * @return die Liste der Talente auf die man den Stufenbonus verteilen
     *      darf.
     */
    public final List< Talent > getTalente() {
        return new ArrayList< Talent >( talente );
    }

    /** {@inheritDoc} */
    public final Talentgruppe getTalentgruppe() {
        return talente.get( 0 ).getTalentgruppe();
    }

    /** {@inheritDoc} */
    public final TalentbonusTyp getTyp() {
        return TalentbonusTyp.FREI;
    }

    /** {@inheritDoc} */
    @Override
    public final String toString() {
        return beschreibungString;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_FREI );

        element.setAttribute( ATTR_BONUS, String.valueOf( stufenBonus ) );

        for ( final Talent talent : talente ) {

            final IXMLElement talentElement = new XMLElement( TAG_TALENT );

            talentElement.setContent( talent.getId() );

            element.addChild( talentElement );
        }

        return element;
    }
}
