/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.kosten;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_KOSTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_RASSE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOSTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOSTEN_RASSE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_NORMAL;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_RASSE;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;

/**
 * Fuer Heldenbausteine, deren Kosten fuer Helden einer bestimmten Rasse
 * anders sind.
 *
 * @author Bernhard Jung
 */
public class KostenRasse implements Kosten {

    /** Die GP-Kosten die normalerweise zu zahlen sind. */
    private final int gpKosten;

    /** Die GP-Kosten, die die spezielle Rasse zu zahlen hat. */
    private final int gpKostenRasse;

    /** Die Rasse die die veraenderten GP-Kosten zahlen muss. */
    private final Rasse rasse;

    /**
     * @param kostenNormal die normalerweise gezahlt werden muessen.
     * @param dieRasse die veraenderte Kosten zahlen muss.
     * @param kostenRasse Die Kosten die die spezielle Rasse zahlen muss.
     */
    public KostenRasse(
            final int kostenNormal,
            final Rasse dieRasse,
            final int kostenRasse ) {

        gpKosten = kostenNormal;

        gpKostenRasse = kostenRasse;

        rasse = dieRasse;
    }

    /** {@inheritDoc} */
    public final int getGpKosten( final Held held ) {

        if ( rasse.equals( held.getRasse() ) ) {

            return gpKostenRasse;

        } else {

            return gpKosten;

        }
    }

    /**
     * @return die GP-Kosten, die jede andere Rasse zahlen muss.
     */
    public final int getGpKostenNormal() {
        return gpKosten;
    }

    /**
     * @return die GP-Kosten, die die spezielle Rasse zahlen muss.
     */
    public final int getGpKostenRasse() {
        return gpKostenRasse;
    }

    /**
     * @return die Rasse, die andere GP-Kosten zahlen muss.
     */
    public final Rasse getRasse() {
        return rasse;
    }

    public final IXMLElement toXml() {

        final IXMLElement kosten = new XMLElement( TAG_KOSTEN );

        final IXMLElement kostenRasse = new XMLElement( TAG_KOSTEN_RASSE );
        kosten.addChild( kostenRasse );

        final IXMLElement rasseElement = new XMLElement( TAG_RASSE );
        rasseElement.setAttribute( ATTR_RASSE, rasse.getId() );
        rasseElement.setAttribute( ATTR_KOSTEN, String.valueOf( gpKostenRasse ) );

        kostenRasse.addChild( rasseElement );

        final IXMLElement normal = new XMLElement( TAG_NORMAL );
        normal.setContent( String.valueOf( gpKosten ) );

        kostenRasse.addChild( normal );

        return kosten;
    }
}
