/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Eine Formel zu Bestimmung des Gewichts eines Helden.
 * Das Gewicht wird aus der Groesse des Helden besitmmt indem ein Betrag von
 * der Groesse des Helden abgezogen wird.
 *
 * @author Bernhard Jung
 */
public class GewichtFormel {

    /**
     * Die Vorlage fuer die Textrepraesentation der Formel.
     */
    private static final String FORMEL_TEXT = I18n.i18n( "GewichtFormel" );

    /**
     * Der Betrag der von der Groesse des Helden in cm
     * abgezogen wird um das Gewicht zu erhalten.
     */
    private final int abzug;

    /** Die Formel als Text. */
    private final String toString;

    /**
     * @param derAbzug Der Betrag der von der Groesse des Helden in cm
     *      abgezogen wird um das Gewicht zu erhalten.
     */
    public GewichtFormel( final int derAbzug ) {

        abzug = derAbzug;

        toString = String.format( FORMEL_TEXT, abzug );
    }

    /**
     * Berechnet das Gewicht eines Helden, abhaengig von seiner Groesse.
     * Der Nachteil Fettleibig wird bei der Berechnung mit einbezogen.
     *
     * @param held dessen Gewicht berechent werden soll.
     * @return das Gewicht des Helden.
     */
    public final int berechnen( final Held held ) {

        int gewicht = held.getGroesse() - abzug;

        if ( held.hat( VorNachteil.FETTLEIBIG ) ) {

            final float faktor = 1.5F + ( ((float) Math.random()) / 1.0F );

            gewicht = Math.round( gewicht * faktor );
        }

        return gewicht;
    }

    /**
     * @return die cm, die von der Koerpergroesse abgezogen werden.
     */
    public final int getAbzug() {
        return abzug;
    }

    /**
     * @return die Formel als Text.
     */
    @Override
    public String toString() {
        return toString;
    }

    /**
     * @return das XML-Element, das die Formel repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_GEWICHT );

        element.setContent( String.valueOf( abzug ) );

        return element;
    }
}
