/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.voraussetzungen;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Implementiert eine Voraussetzung wie sie bei Professionen ueblich ist.
 * Der Held muss eine Mindeststufe in einer von evtl zwei Eigenschaften
 * besitzen.
 *
 * @author Bernhard Jung
 */
public class VoraussetzungMindeststufe extends VoraussetzungEigenschaft {

    /**
     * In einer dieser Eigenschaften muss der Held die Mindeststufe besitzen.
     */
    private final Eigenschaft[] eigenschaften;

    /** Die Mindeststufe, der kleinste noch erlaubte Wert. */
    private final int mindeststufe;

    /**
     * Ein String der die Voraussetzung beschreibt wie "KK 11" oder
     * "GE oder FF 13"
     */
    private final String beschreibungString;

    /**
     * @param dieEigenschaften in der der Held die Mindeststufe haben muss.
     * @param dieMindeststufe der kleinste noch erlaubte Wert.
     */
    public VoraussetzungMindeststufe(
            final int dieMindeststufe,
            final Eigenschaft ... dieEigenschaften ) {

        eigenschaften = dieEigenschaften;

        mindeststufe = dieMindeststufe;

        beschreibungString = erzeugeBeschreibung();
    }

    private final String erzeugeBeschreibung() {

        final List< String > worte =
            new ArrayList< String >( eigenschaften.length );

        for ( final Eigenschaft eigenschaft : eigenschaften ) {
            worte.add( eigenschaft.getKuerzel() );
        }

        final StringBuilder builder = new StringBuilder();

        builder.append( Strings.erzeugeAufzaehlungMitOder( worte ) );
        builder.append( " \u2265 " );
        builder.append( mindeststufe );

        return  builder.toString();
    }

    /**
     * @return die Eigenschaften auf die sich die Voraussetzung bezieht.
     */
    @Override
    public final Eigenschaft[] getEigenschaften() {

        final int length = eigenschaften.length;

        final Eigenschaft[] kopie = new Eigenschaft[ length ];

        System.arraycopy( eigenschaften, 0, kopie, 0, length );

        return kopie;
    }

    /**
     * @return der kleinste noch erlaubte Wert fuer die Eigenschaften.
     */
    @Override
    public final int getMindeststufe() {
        return mindeststufe;
    }

    /**
     * Prueft ob der Held eine Mindeststufe in einer Eigenschaft erreicht.
     *
     * @param held der ueberprueft werden soll.
     */
    public KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            if ( held.getStufe( eigenschaft ) < mindeststufe ) {
                // Der Held erfuellt die Voraussetzung nicht.

                // Ist es vielleicht schon das zweite mal das er sie nicht
                // erfuellt?
                if ( ! beschreibung.enthaeltKonflikt() ) {

                    // Es ist das erst mal das er sie nicht erfuellt.
                    beschreibung.einfuegen( beschreibungString );
                    // Wir laufen weiter durch die Schleife, vielleicht
                    // erfuellt die zweite Eigenschaft die Voraussetzung noch.
                }

            } else {

                // Der Held erfuellt die Voraussetzung wir koennen abbrechen.
                beschreibung.leeren();
                break;

            }
        }

        return beschreibung;
    }

    /** @return eine Beschreibung der Voraussetzung. */
    @Override
    public final String toString() {
        return beschreibungString;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element =
            new XMLElement( TAG_MINDESTSTUFE );

        element.setAttribute( ATTR_MINDESTSTUFE,
                              String.valueOf( mindeststufe ) );

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            final IXMLElement eigenschaftElement =
                new XMLElement( TAG_EIGENSCHAFT );

            eigenschaftElement.setContent( eigenschaft.getId() );

            element.addChild( eigenschaftElement );
        }

        return element;
    }
}
