/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.voraussetzungen;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Eine Voraussetzung fuer die Sozialstatus eine Profession.
 * Der SO muss immer zwischen 2 Werten liegen.
 *
 * @author Bernhard Jung
 *
 */
public class VoraussetzungSo extends VoraussetzungEigenschaft {

    /** Der kleinste noch erlaubte Wert. */
    private final int hoechststufe;

    /** Der groesste noch erlaubte Wert. */
    private final int mindeststufe;

    /** Eine kurze Beschreibung der Voraussetzung */
    private final String beschreibungString;

    /**
     * @param dieMindeststufe der kleinste noch erlaubte Wert.
     * @param dieHoechststufe der groesste noch erlaubte Wert.
     */
    public VoraussetzungSo(
            final int dieMindeststufe,
            final int dieHoechststufe ) {

        hoechststufe = dieHoechststufe;

        mindeststufe = dieMindeststufe;

        beschreibungString = String.format(
                I18n.i18n( "VoraussetzungSo" ),
                Eigenschaft.SO.getKuerzel(),
                mindeststufe,
                hoechststufe );
    }

    /** @return new Eigenschaft[]{ Eigenschaft.SO } */
    @Override
    public Eigenschaft[] getEigenschaften() {
        return new Eigenschaft[]{ Eigenschaft.SO };
    }

    /** @return die hoechste noch erlaubte Stufe fuer den SO eines Helden. */
    @Override
    public int getHoechststufe() {
        return hoechststufe;
    }

    /** @return die kleinste noch erlaubte Stufe fuer den SO eines Helden. */
    @Override
    public int getMindeststufe() {
        return mindeststufe;
    }

    /** @return {@code true} */
    @Override
    public boolean hatHoechststufe() {
        return true;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final int so = held.getStufe( Eigenschaft.SO );

        if ( ( so < mindeststufe ) || ( hoechststufe < so ) ) {

            beschreibung.einfuegen( beschreibungString );
        }

        return beschreibung;
    }

    /** @return eine kurze Beschreibung der Voraussetzung. */
    @Override
    public final String toString() {
        return beschreibungString;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_SO );

        element.setAttribute( ATTR_MINDESTSTUFE,
                              String.valueOf( mindeststufe ) );

        element.setAttribute( ATTR_HOECHSTSTUFE,
                              String.valueOf( hoechststufe ) );

        return element;
    }
}
