/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Erlaubt es die Talente zu waehlen. Die Stufen der Talente koennen
 * geaendert werden und man kann Talente (de)aktivieren.
 * Die Schnittstelle besteht aus mehreren Karteikarten, welche die Talente der
 * einzelnen Talentgruppen anzeigen.
 *
 * @author Bernhard Jung
 */
public class PanelTalente extends JPanel implements Observer {

    /** Der Text neben der Combobox mit der man die Lernmethode waehlen kann. */
    private static final String LERNMETHODE =
        I18n.i18n( "PanelTalente.Lernmethode" );

    /** Die maximal Anzahl an Talenten die aktiviert werden darf. */
    public static final int MAX_AKTIVIEREN = 5;

    /**
     * Ein Text der dem Benutzer mitteilt, dass er genau ein Talent aktiviert
     * hat.
     */
    private static final String TALENT_AKTIVIERT =
        I18n.i18n( "PanelTalente.TalentAktiviert" );

    /**
     * Ein Text der dem Benutzer mitteilt wie viele Talent von ihm aktiviert
     * wurden.
     */
    private static final String TALENTE_AKTIVIERT =
        I18n.i18n( "PanelTalente.TalenteAktiviert" );

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 5 Talente aktivert werden
     * duerfen.
     */
    public static final String ZU_VIELE_AKTIVIERT =
        I18n.i18n( "PanelTalente.ZuVieleAktiviert" );

    /**
     * Das Label mit dem angezeigt wird wie viele Talente der Benutzer schon
     * aktiviert hat.
     */
    private final JLabel aktivierteTalente;

    /** Die Liste der Kampftechniken. */
    private PanelKampftechnikenListe kampftechniken;

    /**
     * Die Talente der einzelnen Talentgruppen mit Ausnahme der Talentgruppe
     * Kampf (siehe oben). Fuer jede Talentgruppe gibt es ein eigenes
     * PanelTalente.
     */
    private final Collection< PanelTalenteListe > talente;

    /** Erzeugt ein neues Panel. */
    public PanelTalente( final boolean zumSteigern ) {

        aktivierteTalente = new JLabel( TALENT_AKTIVIERT );

        aktivierteTalente.setHorizontalAlignment( SwingConstants.CENTER );

        talente = new ArrayList< PanelTalenteListe >();

        aufbauenBenutzeroberflaeche( zumSteigern );
    }

    /** Baut die Benutzeroberflaeche auf. */
    private final void aufbauenBenutzeroberflaeche( final boolean zumSteigern ){

        final JComboBox lernmethode = new JComboBox( Lernmethode.values() );

        final Lernmethoden lernmethoden = new Lernmethoden( lernmethode );

        kampftechniken =
            new PanelKampftechnikenListe( zumSteigern, lernmethoden );

        final JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.setTabPlacement( JTabbedPane.LEFT );

        // Ein Extra Panel fuer die Kampftechniken.
        SimpleInternalFrame frame;
        frame = new SimpleInternalFrame(
                "<html><b>" + Talentgruppe.KAMPF.getName() );

        frame.setContent( kampftechniken );

        tabbedPane.addTab( Talentgruppe.KAMPF.getKuerzel(), frame );

        for ( final Talentgruppe gruppe : Talentgruppe.getTalentgruppenAlle() ) {

            if ( Talentgruppe.KAMPF == gruppe ) {
                // Fuer die Kampftechniken haben wir ein extra Panel verwendet.
                // siehe oben
                continue;
            }

            final PanelTalenteListe panelTalente =
                new PanelTalenteListe( gruppe, zumSteigern, lernmethoden );

            talente.add( panelTalente );

            frame = new SimpleInternalFrame( "<html><b>" + gruppe.getName() );

            frame.setContent( panelTalente );

            tabbedPane.addTab( gruppe.getKuerzel(), frame );
        }

        if ( zumSteigern ) {

            setLayout( new FormLayout(
                    "0px:G, P, 2dlu, P, 0px:G",
                    "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( new JLabel( LERNMETHODE + ":" ), cc.xy( 2, 2 ) );
            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new FormLayout( "F:0px:G", "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( aktivierteTalente, cc.xy( 1, 2 ) );

            add( tabbedPane, cc.xy( 1, 4 ) );
        }
    }

    /**
     * Bestimmt den Helden dessen Werte angezeigt werden.
     *
     * @param held dessen Werte angezeigt werden sollen.
     */
    public final void setHeld( final Held held ) {

        held.addObserver( this );

        kampftechniken.setHeld( held );

        for ( final PanelTalenteListe panel : talente ) {

            panel.setHeld( held );

        }

        update( held, null );
    }

    /**
     * Wir aufgerufen wenn am ueberwachten Helden eine Veraenderung
     * durchgefuehrt wird.
     *
     * @param o der Held der ueberwacht wird.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        final Held held = (Held) o;

        final int anzahlAktivierteTalente = held.getAnzahlAktivierteTalente();

        final String text;

        if ( 1 == anzahlAktivierteTalente ) {

            text = TALENT_AKTIVIERT;

        } else {

            text = String.format( TALENTE_AKTIVIERT, anzahlAktivierteTalente );

        }

        if ( anzahlAktivierteTalente > MAX_AKTIVIEREN ) {

            aktivierteTalente.setForeground( Util.FEHLERFARBE );
            aktivierteTalente.setToolTipText(
                    "<html>" + ZU_VIELE_AKTIVIERT );

        } else {

            aktivierteTalente.setForeground( getForeground() );
            aktivierteTalente.setToolTipText( null );

        }

        aktivierteTalente.setText( text );
    }
}
