/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.KeyStroke;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import de.bernhardjung.dsaprogramm.Start;

/**
 * Ein kleine Sammlung von Konstanten und statischen Methoden die in der
 * Benutzeroberflaeche verwendet werden.
 *
 * @author Bernhard Jung
 */
public class Util {

    /** Die Farbe die zum Markieren von Regelproblemen verwendet wird. */
    public static final Color FEHLERFARBE = Color.RED;

    /** Erlaubt es einen JSpinner mit dem Mausrad zu bedienen. */
    private static final MyMouseWheelListener SPINNER_MOUSE_WHEEL_LISTENER =
        new MyMouseWheelListener();

    /** Eine Schriftart die als Ueberschrift verwendet werden kann. */
    private static final Font UEBERSCHRIFT;

    /**
     * Der Wert der fuer das Attribut Unit Increment eines
     * {@link JScrollBar} verwendet wird.
     */
    public static final int UNIT_INCREMENT = 30;

    static {
        UEBERSCHRIFT = new JLabel().getFont().deriveFont( Font.BOLD );
    }

    /**
     * @return ein Label mit rechtsbuendig ausgerichtetem Text.
     */
    public static final JLabel erzeugeLabelRechtsbuendig() {

        final JLabel label = new JLabel();

        label.setHorizontalAlignment( SwingConstants.RIGHT );

        return label;
    }

    /**
     * Pakt einen Componente in einer Scrollpane.
     *
     * @param component
     *            die in die Scrollpane soll.
     * @return die Scrollpane mit der Componente.
     */
    public static final JScrollPane erzeugeScrollPane(
            final JComponent component ) {

        final JScrollPane scrollPane = new JScrollPane( component );

        // Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( component.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        final Dimension groesse = scrollPane.getPreferredSize();

        groesse.height = 200;

        scrollPane.setPreferredSize( groesse );

        return scrollPane;
    }

    /**
     * Erzeugt einen JSpinner, der mit dem Mausrad bedient werden kann.
     *
     * @param wert der Startwert des Spinners.
     * @param minimum der kleinste noch erlaubte Wert.
     * @param maximum der groesste noch erlaubte Wert.
     * @param schrittweite fuer den Spinner.
     * @return einen Spinner
     */
    public static final JSpinner erzeugeSpinner(
            final double wert,
            final double minimum,
            final double maximum,
            final double schrittweite ) {

        final SpinnerModel model = new SpinnerNumberModel(
                wert,
                minimum,
                maximum,
                schrittweite );

        final JSpinner spinner = new JSpinner( model );

        spinner.addMouseWheelListener( SPINNER_MOUSE_WHEEL_LISTENER );

        return spinner;
    }

    /**
     * Erzeugt einen JSpinner, der mit dem Mausrad bedient werden kann.
     *
     * @param wert der Startwert des Spinners.
     * @param minimum der kleinste noch erlaubte Wert.
     * @param maximum der groesste noch erlaubte Wert.
     * @param schrittweite fuer den Spinner.
     * @return einen Spinner
     */
    public static final JSpinner erzeugeSpinner(
            final int wert,
            final int minimum,
            final int maximum,
            final int schrittweite ) {

        final SpinnerModel model = new SpinnerNumberModel(
                wert,
                minimum,
                maximum,
                schrittweite );

        final JSpinner spinner = new JSpinner( model );

        spinner.addMouseWheelListener( SPINNER_MOUSE_WHEEL_LISTENER );

        return spinner;
    }

    /**
     * @param text fuer die Ueberschrift.
     * @return ein als Ueberschrift formatiertes JLabel.
     */
    public static final JLabel erzeugeUeberschrift( final String text ) {

        final JLabel ueberschrift = new JLabel( text );

        ueberschrift.setFont( UEBERSCHRIFT );

        return ueberschrift;
    }

    /**
     * Liefert zu einem Buchstaben die Konstante aus der Klasse
     * {@link KeyEvent}. Zu &quot;A&quot; z.B. {@link KeyEvent#VK_A}.
     *
     * @param buchstabe zu dem die Konstante gesucht ist.
     * @return die Konstante aus {@link KeyEvent}.
     */
    public static final int getKeyCode( final String buchstabe ) {
        return KeyStroke.getKeyStroke( buchstabe ).getKeyCode();
    }

    /**
     * Laedt ein png-Icon aus dem Ordner bilder.
     *
     * @param name der Name der Datei wird vorne mit 'bilder/' und hinten
     *      mit '.png' ergaenzt.
     *      <dl><dt>Beispiel:</dt><dd>
     *      Mit name='icon' wird die Datei 'bilder/icon.png' geladen.
     *      </dd></dl>
     *
     * @return ein Bild.
     */
    public static final ImageIcon ladeIcon( final String name ) {

        final URL url = Start.class.getResource( "bilder/" + name + ".png");

        if ( null != url ) {

            return new ImageIcon( url );

        } else {

            throw new IllegalArgumentException( String.format(
                    "Das Icon %1$s konnte nicht gefunden werden.", name ) );
        }
    }

    /**
     * Ein MouseWheelListener der es erlaubt einen JSpinner, der einen Integer
     * Wert darstellt, mit dem Mausrade zu bedienen.
     *
     * @author Bernhard Jung
     */
    private static class MyMouseWheelListener implements MouseWheelListener {
        public final void mouseWheelMoved(final MouseWheelEvent e) {

            final JSpinner spinner = (JSpinner) e.getSource();

            if ( ! spinner.isEnabled() ) {
                // Der Spinner ist nicht aktiv. Abbruch.
                return;
            }

            final SpinnerNumberModel model;
            model = (SpinnerNumberModel) spinner.getModel();

            if ( e.getWheelRotation() > 0 ) {

                final Object previousValue = model.getPreviousValue();

                if ( null != previousValue ) {

                    model.setValue( previousValue );
                }

            } else {

                final Object nextValue = model.getNextValue();

                if ( null != nextValue ) {

                    model.setValue( nextValue );
                }
            }
        }
    }
}
