/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.FocusManager;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.FileFilter;

abstract class ActionSpeichern extends AbstractAction {

    /**
     * Die Meldung die dem Benutzer angezeigt wird, wenn die Datei in die
     * das Heldendokument gespeichert werden soll schon existiert. Die Meldung
     * fragt ob der Benutzer die Datei ersetzen will.
     */
    private static final String DATEI_EXISTIERT_MELDUNG =
        I18n.i18n( "ActionSpeichern.DateiExistiert.Meldung");

    /**
     * Der Titel fuer die Meldung wenn die Datei fuer das Heldendokument schon
     * existiert.
     */
    private static final String DATEI_EXISTIERT_TITEL =
        I18n.i18n( "ActionSpeichern.DateiExistiert.Titel");

    /**
     * Die Meldung die dem Benutzer mitteilt wenn die Datei fuer das
     * Heldendokument nicht angelegt werden kann.
     */
    private static final String DATEI_NICHT_ANGELEGT_MELDUNG =
        I18n.i18n(  "ActionSpeichern.DateiNichtAngelegt.Meldung" );

    /**
     * Titel fuer die Meldung dass die Datei fuer das Heldendokument nicht
     * angelegt werden kann.
     */
    private static final String DATEI_NICHT_ANGELEGT_TITEL =
        I18n.i18n( "ActionSpeichern.DateiNichtAngelegt.Titel" );

    /**
     * Die Meldung die angezeigt wird wenn das Programm keine Schreibrechte
     * fuer die Datei fuer das Heldendokument besitzt.
     */
    private static final String KEINE_SCHREIBRECHTE_MELDUNG =
        I18n.i18n( "ActionSpeichern.KeineSchreibrechte.Meldung" );

    /**
     * Der Titel fuer die Meldung wenn das Programm nicht in die Datei fuer
     * das Heldendokument schreiben kann.
     */
    private static final String KEINE_SCHREIBRECHTE_TITEL =
        I18n.i18n( "ActionSpeichern.KeineSchreibrechte.Titel");

    private final JFileChooser fileChooser;

    protected final JFrame hauptfenster;

    protected final Hauptpanel hauptpanel;

    /**
     * @param nameId
     *            die Id, aus der Lokalisierungsdatei, fuer den Namen.
     * @param mnemonicId
     *            die Id, aus der Lokalisierungsdatei, fuer das Mnemonic.
     * @param dasHauptfenster
     *            der Anwendung.
     */
    ActionSpeichern(
            final String nameId,
            final String mnemonicId,
            final JFrame dasHauptfenster,
            final Hauptpanel dasHauptpanel ) {

        super( I18n.i18n( nameId ) );

        final String mnemonic = I18n.i18n( mnemonicId );
        putValue( MNEMONIC_KEY, Util.getKeyCode( mnemonic ) );

        hauptfenster = dasHauptfenster;

        hauptpanel = dasHauptpanel;

        fileChooser = erzeugeFileChooser();
    }

    /**
     * @param nameId
     *            die Id, aus der Lokalisierungsdatei, fuer den Namen.
     * @param icon
     *            der Name des Icons.
     * @param mnemonicId
     *            die Id, aus der Lokalisierungsdatei, fuer das Mnemonic.
     * @param tastenId
     *            die Id, aus der Lokalisierungsdatei, fuer das Tastenkuerzel.
     * @param dasHauptfenster
     *            der Anwendung.
     */
    ActionSpeichern(
            final String nameId,
            final String icon,
            final String mnemonicId,
            final String tastenId,
            final JFrame dasHauptfenster,
            final Hauptpanel dasHauptpanel ) {

        super( I18n.i18n( nameId ), Util.ladeIcon( icon ) );

        final String mnemonic = I18n.i18n( mnemonicId );
        putValue( MNEMONIC_KEY, Util.getKeyCode( mnemonic ) );

        final String tasten = I18n.i18n( tastenId );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( tasten ) );

        hauptfenster = dasHauptfenster;

        hauptpanel = dasHauptpanel;

        fileChooser = erzeugeFileChooser();
    }

    /**
     * @param nameId
     *            die Id, aus der Lokalisierungsdatei, fuer den Namen.
     * @param icon
     *            der Name des Icons.
     * @param mnemonicId
     *            die Id, aus der Lokalisierungsdatei, fuer das Mnemonic.
     * @param tastenId
     *            die Id, aus der Lokalisierungsdatei, fuer das Tastenkuerzel.
     * @param dasHauptfenster
     *            der Anwendung.
     * @param derFileChooser
     *            zum Auswaehlen der Datei in die gespeichert wird.
     */
    ActionSpeichern(
            final String nameId,
            final String icon,
            final String mnemonicId,
            final String tastenId,
            final JFrame dasHauptfenster,
            final Hauptpanel dasHauptpanel,
            final JFileChooser derFileChooser ) {

        super( I18n.i18n( nameId ), Util.ladeIcon( icon ) );

        final String mnemonic = I18n.i18n( mnemonicId );
        putValue( MNEMONIC_KEY, Util.getKeyCode( mnemonic ) );

        final String tasten = I18n.i18n( tastenId );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( tasten ) );

        hauptfenster = dasHauptfenster;

        hauptpanel = dasHauptpanel;

        fileChooser = derFileChooser;
    }

    public void actionPerformed( final ActionEvent e ) {

        // Damit der Name der evtl. gerade ins Textfeld eingegeben wurde
        // uebernommen wird.
        FocusManager.getCurrentManager().clearGlobalFocusOwner();

        final String dateiname = getDateinameVorschlag();

        final File verzeichnis = new File( getVerzeichnis() );

        final File datei = new File( verzeichnis, dateiname );

        fileChooser.setSelectedFile( datei );

        final int speichern = fileChooser.showDialog( hauptfenster,
                                             (String) getValue( NAME ) );

        if ( speichern == JFileChooser.APPROVE_OPTION ) {

            File gewaehlteDatei = fileChooser.getSelectedFile();

            if ( null == gewaehlteDatei ) {
                return;
            }

            String testname = gewaehlteDatei.getName();

            testname = testname.toLowerCase();

            if ( ! testname.endsWith( getDateierweiterung() ) ) {
                // Die Datei hat die falsche Endung.
                // Die richtige wird angehaengt.

                String pfad = gewaehlteDatei.getAbsolutePath();

                pfad = pfad + getDateierweiterung();

                gewaehlteDatei = new File( pfad );
            }

            if ( ! gewaehlteDatei.exists() ) {

                final boolean dateiErzeugt = erzeugeDatei( gewaehlteDatei );

                if (!dateiErzeugt) {

                    JOptionPane.showMessageDialog( hauptfenster,
                            DATEI_NICHT_ANGELEGT_MELDUNG,
                            DATEI_NICHT_ANGELEGT_TITEL,
                            JOptionPane.ERROR_MESSAGE );

                    return;
                }

            } else {

                final int ersetzen = JOptionPane.showConfirmDialog(
                        hauptfenster,
                        String.format( DATEI_EXISTIERT_MELDUNG,
                                       gewaehlteDatei.getName() ),
                        DATEI_EXISTIERT_TITEL,
                        JOptionPane.WARNING_MESSAGE );

                if (ersetzen == JOptionPane.NO_OPTION) {
                    return;
                }
            }

            if ( ! gewaehlteDatei.canWrite() ) {

                JOptionPane.showMessageDialog(
                        hauptfenster,
                        KEINE_SCHREIBRECHTE_MELDUNG,
                        KEINE_SCHREIBRECHTE_TITEL,
                        JOptionPane.ERROR_MESSAGE );

                return;
            }

            // Speichert das Dokument
            speichern( gewaehlteDatei );

            final File gewaehltesVerzeichnis = gewaehlteDatei.getParentFile();

            if ( null != gewaehltesVerzeichnis ) {

                setVerzeichnis( gewaehltesVerzeichnis );
            }
        }
    }

    private final boolean erzeugeDatei( final File datei ) {

        boolean dateiErzeugt = false;

        try {

            dateiErzeugt = datei.createNewFile();

        } catch (final IOException e) {
        }

        return dateiErzeugt;
    }

    private JFileChooser erzeugeFileChooser() {

        final JFileChooser fileChooser = new JFileChooser();

        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setMultiSelectionEnabled( false );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.setFileFilter( new FileFilter( getDateierweiterung() ) );

        return fileChooser;
    }

    /**
     * @return die Dateierweiterung wie '.xml' oder '.rtf'.
     */
    abstract String getDateierweiterung();

    /**
     * @return einen moeglichen Dateinamen fuer die zu speichernde Datei.
     */
    abstract String getDateinameVorschlag();

    /**
     * @return das Verzeichnis, das im Filechooser geoeffnet werden soll.
     */
    abstract String getVerzeichnis();

    /**
     * @param datei in die gespeichert werden soll.
     */
    abstract void speichern( final File datei );

    /**
     * @param verzeichnis
     *            das im Filechooser geoeffnet werden soll.
     */
    abstract void setVerzeichnis( final File verzeichnis );
}
