/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.text.Collator;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Stellt eine Liste von Sonderfertigkeiten dar.
 * Jede Sonderfertigkeit wird genau einer Gruppe zugeordnet.
 * Manche Gruppen bestehen nur aus einer Sonderfertigkeit, wie z.B. die
 * Kulturkunde.
 *
 * @author Bernhard Jung
 *
 */
public class Sonderfertigkeitgruppe
extends AbstractList< Sonderfertigkeit >
implements Comparable< Sonderfertigkeitgruppe >{

    /** Ein Collator zum Sortiern der Gruppen nach Namen. */
    private static final Collator COLLATOR = Collator.getInstance();

    /** Eine Liste aller Gruppen */
    private static final List< Sonderfertigkeitgruppe > GRUPPEN =
        new ArrayList< Sonderfertigkeitgruppe >();

    public static final Sonderfertigkeitgruppe ALLGEMEIN =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Allgemein" );
/*
    public static final Sonderfertigkeitgruppe ELFENLIEDER =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Elfenlieder" );
*//*
    public static final Sonderfertigkeitgruppe GELAENDEKUNDE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Gelaendekunde" );
*/
    public static final Sonderfertigkeitgruppe KAMPF =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Kampf" );
/*
    public static final Sonderfertigkeitgruppe KULTURKUNDE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Kulturkunde" );
*/
    public static final Sonderfertigkeitgruppe MAGISCHE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Magische" );
/*
    public static final Sonderfertigkeitgruppe SCHARFSCHUETZE =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Scharfschuetze" );

    public static final Sonderfertigkeitgruppe STABZAUBER =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Stabzauber" );
*/
    public static final Sonderfertigkeitgruppe TALENTSPEZIALISIERUNG =
        new Sonderfertigkeitgruppe(
                "SonderfertigkeitGruppe.Talentspezialisierung" );

    public static final Sonderfertigkeitgruppe WAFFENSPEZIALISIERUNG =
        new Sonderfertigkeitgruppe( "SonderfertigkeitGruppe.Waffenspezialisierng" );

    /**
     * @return eine Liste aller Gruppen.
     */
    public static final List< Sonderfertigkeitgruppe > getGruppen() {
        return new ArrayList< Sonderfertigkeitgruppe >( GRUPPEN );
    }

    /** Die Id der Sonderfertigkeit. */
    private final String id;

    /**
     * Eine kuerzere Form des Namens der Sonderfertigkeit, zB "Talentsp.".
     */
    private final String kuerzel;

    /** Der volle Name der Sonderfertigkeit, zB "Talentspezialisierung" */
    private final String name;

    /** Die Sonderfertigkeiten die zu dieser Liste gehoeren. */
    private final List< Sonderfertigkeit > sonderfertigkeiten;

    private Sonderfertigkeitgruppe( final String dieId ) {

        id = dieId;

        name = I18n.i18n( id + ".Name" );

        kuerzel = I18n.i18n( id + ".Kuerzel" );

        sonderfertigkeiten = new ArrayList< Sonderfertigkeit >();

        int index = Collections.binarySearch( GRUPPEN, this );

        index = -( index + 1);

        GRUPPEN.add( index, this );
    }

    /**
     * Sortiert die Gruppen alphabetisch nach Namen.
     */
    public int compareTo( final Sonderfertigkeitgruppe o ) {
        return COLLATOR.compare( name, o.name );
    }

    /**
     * Fuegt der Gruppe eine neue Sonderfertigkeit hinzu.
     * Macht nichts, wenn die Sonderfertigkeit schon in der Gruppe ist.
     *
     * @param sonderfertigkeit fuer die Gruppe.
     */
    final void einfuegen( final Sonderfertigkeit sonderfertigkeit ) {

        int index = Collections.binarySearch( sonderfertigkeiten,
                                              sonderfertigkeit );

        if ( 0 > index ) {

            index = -( index + 1 );

            sonderfertigkeiten.add( index, sonderfertigkeit );
        }
    }

    /**
     * @return die Sonderfertigkeit an Position {@code index}.
     */
    @Override
    public Sonderfertigkeit get( final int index ) {
        return sonderfertigkeiten.get( index );
    }

    /**
     * @return den Namen der Gruppe.
     */
    public final String getName() {
        return name;
    }

    /**
     * @return eine Abkuerzung fuer den Namen der Gruppe.
     */
    public final String getKuerzel() {
        return kuerzel;
    }

    /**
     * @return die Anzahl an Sonderfertigkeiten in der Liste.
     */
    @Override
    public int size() {
        return sonderfertigkeiten.size();
    }
}
