/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Eine Voraussetzung bei der in Held eine Eigenschaft auf einer Mindeststufe
 * besitzen muss.
 * 
 * @author Bernhard Jung
 */
class VoraussetzungEigenschaft implements Voraussetzung {
    
    /** Beschreibt dem Benutzer die Voraussetzung. */
    private final String beschreibungString;
    
    /** Die Eigenschaft in welcher der Held die Mindesstufe erreichen muss. */
    private final Eigenschaft eigenschaft;
    
    /** 
     * Die Mindeststufe die der Held mindestens in der Eigenschaft besitzen 
     * muss.
     */
    private final int mindeststufe;
    
    /**
     * @param dieEigenschaft in welcher der Held die Mindeststufe erreichen 
     *      muss.
     * @param dieMindeststufe die der Held erreichen muss.
     */
    VoraussetzungEigenschaft(
            final Eigenschaft dieEigenschaft,
            final int dieMindeststufe ) {
        
        eigenschaft = dieEigenschaft;
        
        mindeststufe = dieMindeststufe;
        
        beschreibungString = String.format(
                I18n.i18n( "VoraussetzungEigenschaft" ),
                eigenschaft.getKuerzel(),
                mindeststufe );
    }
    
    /** @return die Eigenschaft. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) eigenschaft );
    }
    
    /**
     * Prueft ob der Held die Mindeststufe in der Eigenschaft erreicht.
     */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final SonderfertigkeitWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        final int stufe = held.getStufe( eigenschaft );
        
        if ( stufe < mindeststufe ) {
            
            beschreibung.einfuegen( beschreibungString );
            
        }
        
        return beschreibung;
    }

}
