/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Prueft die Voraussetzungen die fuer jedes Talent erfuellt sein muessen.
 * Das waeren:
 * <ul>
 * <li>
 *     Ist das Talent ein Basis-Talent oder vorgegeben muss der Held es
 *     besitzen.
 * </li>
 * <li>
 *     Die Stufe des Talents darf nicht unter der Mindeststufe liegen.
 * </li>
 * <li>
 *     Die Stufe des Talents darf nicht ueber der Hoechststufe liegen.
 * </li>
 * </ul>
 * @author Bernhard Jung
 *
 */
class VoraussetzungStandard extends Voraussetzung {

    /**
     * Beschreibt das die Stufe eines Talents zwischen zwei Grenzwerten liegen
     * muss. Der String wird mit {@link String#format(String, Object[])}
     * verarbeitet.<br>
     * Beispiel: 'Voraussetzung %1$s &le; TaW &le; %2$s'
     */
    private static final String GRENZEN =
        I18n.i18n( "VoraussetzungStandard.Talent.Grenzen" );

    /**
     * Beschreibt das der Held ein Talent besitzen muss.
     */
    private static final String VORGEGEBEN =
        I18n.i18n( "VoraussetzungStandard.Talent.Vorgegeben" );

    /**
     * @return leere Liste.
     */
    @Override
    @SuppressWarnings("unchecked")
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final Talent talent ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.hat( talent ) ) {

            // Grenzwerte testen
            final int mindeststufe;
            if ( held.hatVorgegeben( talent ) ) {

                mindeststufe = held.getMindeststufe( talent );

            } else {

                mindeststufe = 0;

            }

            final int hoechststufe = talent.getHoechststufe( held );

            final int stufe = held.getStufe( talent );

            if ( ( stufe < mindeststufe ) || ( hoechststufe < stufe ) ) {

                beschreibung.einfuegen(
                        String.format( GRENZEN, mindeststufe, hoechststufe ) );
            }


        } else {

            // Der Held besitzt das Talent nicht. Deswegen sind wir in diesem
            // else Teil gelandet ...
            if ( held.hatVorgegeben( talent ) ) {
                // ... er muss es aber besitzen
                beschreibung.einfuegen( VORGEGEBEN );
            }
        }

        return beschreibung;
    }

}
