/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.stufen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;

/**
 * Liefert die Mindest und Hoechststufen, sowie die Schrittweite fuer einen
 * Vorteil mit Stufen.
 *
 * @author Bernhard Jung
 */
public interface Stufen {

    /** Die Hoechststufe ist gleich dem SO des Helden. */
    public static final Stufen AUSRUESTUNGSVORTEIL =
        new StufenAusruestungsvorteil();

    /** Fuer Vor-/Nachteile ohne Stufen. */
    public static final Stufen KEINE = new StufenKeine();

    /** Stufen von 1 bis 3, Schrittweite 1. */
    public static final Stufen STUFE_1_BIS_3 = new StufenStandard( 1, 3, 1 );

    /** Stufen von 1 bis 5, Schrittweite 1. */
    public static final Stufen STUFE_1_BIS_5 = new StufenStandard( 1, 5, 1 );

    /** Stufen von 1 bis 6, Schrittweite 1. */
    public static final Stufen STUFE_1_BIS_6 = new StufenStandard( 1, 6, 1 );

    /** Stufen von 3 bis 12, Schrittweite 1. */
    public static final Stufen STUFE_3_BIS_12 = new StufenStandard( 3, 12, 1 );

    /**
     * Stufen von 5 bis 12, Schrittweite 1. Die typische Schlechte Eigenschaft.
     */
    public static final Stufen STUFE_5_BIS_12 = new StufenStandard( 5, 12, 1 );

    /**
     * Stufen von 6 bis 12, Schrittweite 2. Die andere typische Schlechte
     * Eigenschaft.
     */
    public static final Stufen STUFE_6_BIS_12 = new StufenStandard( 6, 12, 2 );

    /** Stufen von 1 bis unendlich, Schrittweite 1. */
    public static final Stufen STUFE_1_BIS_MAX =
        new StufenStandard( 1, Integer.MAX_VALUE, 1 );

    /** Die Stufen fuer den Vorteil Verbindungen. */
    public static final Stufen VERBINDUNGEN = new StufenVerbindungen();

    /**
     * @return die Regelelemente (Eigenschaften, Vor-/Nachteile usw.) die bei
     *         der Bestimmung der Stufe wichtig sind.
     */
    public Collection< Object > getAbhaengigkeiten();

    /**
     * Liefert den groessten noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils.
     *
     * @param held fuer den die Hoechststufe gesucht ist.
     * @return den groessten noch erlaubten Wert.
     */
    public int getHoechststufe( final Held held );

    /**
     * Liefert den kleinsten, noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils. Der Wert kann vom Helden abhaengen, daher gibt es noch
     * die Methode {@link #getMindeststufe(Held)}.
     *
     * @see #getMindeststufe(Held)
     * @return den kleinsten noch erlaubten Wert.
     */
    public int getMindeststufe();

    /**
     * Liefert den kleinsten, noch erlaubten Wert fuer die Stufe des
     * Vor-/Nachteils.
     * TODO Die Methode sollte auch die durch Rasse, Kultur und Profession vorgegebene Mindeststufe beachten. Die Voraussetzungen sind entsprechend anzupassen.
     * @param held fuer den die Mindeststufe sein soll.
     * @return den kleinsten noch erlaubten Wert.
     */
    public int getMindeststufe( final Held held );

    /**
     * @return die Schrittweite fuer die Stufe.
     */
    public int getSchrittweite();

    /**
     * @return {@code true} wenn der Vor-/Nachteil Stufen hat, sonst
     *      {@code false}.
     */
    public boolean hatStufen();
}
