/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenTalentgruppe;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Implementiert die unterschiedlichen Voraussetzungen fuer den Vorteil
 * Begabung/Unfaehigkeit fuer [Talentgruppe]. Der Held wird dabei auf zu der
 * Begabung/Unfaehigkeit inkompatible Begabungen und Unfaehigkeiten ueberprueft.
 * 
 * @author Bernhard Jung
 */
class VoraussetzungBegabungUnfaehigkeitTalentgruppe extends Voraussetzung {


    /** Begabung oder Unfaehigkeit fuer [Talentgruppe]. */
    private final VorNachteil inkompatiblerVorNachteil;
    
    /**
     * @param derInkompatibleVorNachteil
     *            Soll die Voraussetzung fuer Begabung fuer [Talentgruppe] sein,
     *            so muss hier Unfaehigkeit fuer [Talentgruppe] uebergeben
     *            werden. Soll die Voraussetzung fuer Unfaehigkeit fuer
     *            [Talentgruppe] sein, so muss hier Begabung fuer [Talentgruppe]
     *            uebergeben werden.
     */
    VoraussetzungBegabungUnfaehigkeitTalentgruppe(
            final VorNachteil derInkompatibleVorNachteil ) {

        if ( ( derInkompatibleVorNachteil 
                                != VorNachteil.BEGABUNG_TALENTGRUPPE ) 
          && ( derInkompatibleVorNachteil 
                                != VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE ) ) {
            
            throw new IllegalArgumentException( String.format(
                    "Es durfen nur Begbung/Unfaehigkeit fuer [Talentgruppe] " +
                    "uebergeben werden. Es wurde aber %1$s uebergeben", 
                    derInkompatibleVorNachteil.getName() ) );
        }
        
        inkompatiblerVorNachteil = derInkompatibleVorNachteil;
    }

    /** 
     * @return Begabung Talent/Talentgruppe und Unfaehigkeit 
     *         Talent/Talentgruppe.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( 
                (Object) VorNachteil.BEGABUNG_TALENT,
                VorNachteil.BEGABUNG_TALENTGRUPPE,
                VorNachteil.UNFAEHIGKEIT_TALENT,
                VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE  );
    }

    /**
     * Prueft ob es sich bei einem Talent um eine Nahkampf-Technik handelt und
     * bei einer Talentgruppe um die Gruppe Nahkampffertigkeiten.
     * 
     * @param talent
     *            das ueberprueft werden soll.
     * @param talentgruppe
     *            die ueberprueft werden soll.
     * @return {@code true}, wenn es sich um eine Nahkampf-Technik und die
     *         Talentgruppe Nahkampffertigkeiten handelt, sonst {@code false}.
     */
    private final boolean istNahkampftechnikUndTgNahkampf(
            final Talent talent,
            final Object talentgruppe ) {
        
        return Talentgruppe.KAMPF.equals( talent.getTalentgruppe() )
            && ((Kampftechnik) talent).istNahkampfTechnik()
            && talentgruppe.equals( VariantenTalentgruppe.NAHKAMPF );
    }
    
    /**
     * Prueft ob es sich bei einem Talent um eine Fernkampf-Technik handelt und
     * bei einer Talentgruppe um die Gruppe Fernkampffertigkeiten.
     * 
     * @param talent
     *            das ueberprueft werden soll.
     * @param talentgruppe
     *            die ueberprueft werden soll.
     * @return {@code true}, wenn es sich um eine Fernkkampf-Technik und die
     *         Talentgruppe Fernkampffertigkeiten handelt, sonst {@code false}.
     */
    private final boolean istFernkampftechnikUndTgFernkampf(
            final Talent talent,
            final Object talentgruppe ) {
        
        return Talentgruppe.KAMPF.equals( talent.getTalentgruppe() )
            && ((Kampftechnik) talent).istFernkampfTechnik()
            && talentgruppe.equals( VariantenTalentgruppe.FERNKAMPF );
    }

    /**
     * Prueft ob ein Held Begabung/Unfaehigkeit fuer [Talent] in einer
     * bestimmten Talentgruppe besitzt.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talentgruppe
     *            Die bestimmte Talentgruppe in der eine Begabung/Unfaehigkeit
     *            gesucht wird.
     * @param begabungUnfaehigkeit
     *            der Vor-/Nachteil Begabung/Unfaehigkeit fuer [Talent].
     * @param beschreibung
     *            in die eingetragen wird, wenn der Held eine
     *            Begabung/Unfaehigkeit fuer [Talent] besitzt.
     */
    private final void pruefeBegabungUnfaehigkeitTalent(
            final Held held, 
            final Object talentgruppe, 
            final VorNachteil begabungUnfaehigkeit,
            final KonfliktBeschreibung beschreibung ) {

        for ( final VorNachteilWert wert : 
                                    held.getWerte( begabungUnfaehigkeit ) ) {
            
            final Talent talent = (Talent) wert.getVariante();
            
            if ( talentgruppe.equals( talent.getTalentgruppe() )
              || istNahkampftechnikUndTgNahkampf( talent, talentgruppe )
              || istFernkampftechnikUndTgFernkampf( talent, talentgruppe ) ) {
                
                beschreibung.einfuegen( String.format(
                        VoraussetzungNichtKombinierbar.NICHT_KOMBINIERBAR,
                        wert.getName() ) );
            
            }
        }
    }

    /**
     * @param held
     *            der den {@code wert} waehlen will oder besitzt.
     * @param wert
     *            Eine Begabung fuer [Talentgruppe], die der Held waehlen will
     *            oder besitzt.
     */
    public final KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        final Object talentgruppe = wert.getVariante();
        
        pruefeBegabungUnfaehigkeitTalent( held, 
                                          talentgruppe,
                                          VorNachteil.BEGABUNG_TALENT,
                                          beschreibung );
        
        pruefeBegabungUnfaehigkeitTalent( held, 
                                          talentgruppe,
                                          VorNachteil.UNFAEHIGKEIT_TALENT,
                                          beschreibung );

        pruefeBegabungUnfaehigkeitTalentgruppe( held, talentgruppe, beschreibung );
        
        return beschreibung;
    }

    /**
     * Prueft ob der Held der Vor-/Nachteil {@link #inkompatiblerVorNachteil}
     * mit der Variante {@code talentgruppe} besitzt.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talentgruppe
     *            Die Variante fuer {@link #inkompatiblerVorNachteil}.
     * @param beschreibung
     *            in die eingetragen wird, wenn der Held den inkompatiblen
     *            Vor-/Nachteil besitzt.
     */
    private final void pruefeBegabungUnfaehigkeitTalentgruppe(
            final Held held,
            final Object talentgruppe,
            final KonfliktBeschreibung beschreibung ) {
        
        final VorNachteilWert unfaehigkeit = new VorNachteilWert(
                inkompatiblerVorNachteil, talentgruppe );
        
        if ( held.hat( unfaehigkeit ) ) {

            beschreibung.einfuegen( String.format(
                    VoraussetzungNichtKombinierbar.NICHT_KOMBINIERBAR,
                    unfaehigkeit.getName() ) );
        
        }
    }
}
